@@ -1,6 +1,6 @@
-#' Optimized target quantities in simulation experiments (ProBABLI)
+#' One Dimensional Root (Zero) Finding in Simulation Experiments
 #'
-#' This function provides a stochastic optimization approach to solving
+#' This function provides a stochastic root-finding approach to solving
 #' specific quantities in simulation experiments (e.g., solving for a specific
 #' sample size to meet a target power rate) using the
 #' Probablistic Bisection Algorithm with Bolstering and Interpolations
@@ -10,11 +10,13 @@
 #' fixed, and the constant \code{b} is required in order to
 #' solve the root equation \code{f(x) - b = 0}.
 #'
-#' Optimization is performed using the probabilistic bisection algorithm
-#' (\code{\link{PBA}}) to find the associated root given the noisy simulation
-#' objective function evaluations. Information is also collected throughout
-#' the iterations in order to make more aggressive predictions about the
-#' associated root via interpolation and extrapolation.
+#' Root finding is performed using a progressively bolstered version of the
+#' probabilistic bisection algorithm (\code{\link{PBA}}) to find the
+#' associated root given the noisy simulation
+#' objective function evaluations. Information is collected throughout
+#' the search to make more accurate predictions about the
+#' associated root via interpolation. If interpolations fail, then the last
+#' iteration of the PBA search is returned as the best guess.
 #'
 #' @param design a \code{tibble} or \code{data.frame} object containing
 #'   the Monte Carlo simulation conditions to be studied, where each row
@@ -27,11 +29,13 @@
 #'
 #' @param replications a named list or vector indicating the number of replication to
 #'   use for each design condition per PBA iteration. By default the input is a
-#'   \code{list} with the arguments \code{burnin = 15L}, specifying the number
+#'   \code{list} with the arguments \code{burnin.iter = 15L}, specifying the number
 #'   of burn-in iterations to used, \code{burnin.reps = 100L} to indicate how many
 #'   replications to use in each burn-in iteration, \code{max.reps = 500L} to
-#'   prevent the replications from increasing higher than this number, and
-#'   \code{increase.by = 10L} to indicate how many replications to increase
+#'   prevent the replications from increasing higher than this number,
+#'   \code{min.total.reps = 9000L} to avoid termination when very few replications
+#'   have been explored (lower bound of the replication budget),
+#'   and \code{increase.by = 10L} to indicate how many replications to increase
 #'   after the burn-in stage. Unless otherwise specified these defaults will
 #'   be used, but can be overwritten by explicit definition (e.g.,
 #'   \code{replications = list(increase.by = 25L)})
@@ -43,6 +47,15 @@
 #'   gradually increase to reduce the sampling variability as the PBA approaches
 #'   the root.
 #'
+#' @param method optimizer method to use. Default is the stochastic root-finder
+#'   \code{'ProBABLI'}, but can also be the deterministic options \code{'Brent'}
+#'   (which uses the function \code{\link{uniroot}}) or \code{'bisection'}
+#'   (for the classical bisection method). If using deterministic root-finders then
+#'   \code{replications} must either equal a single constant to reflect
+#'   the number of replication to use per deterministic iteration or be a
+#'   vector of length \code{maxiter} to indicate the replications to use per
+#'   iteration
+#'
 #' @param generate generate function. See \code{\link{runSimulation}}
 #'
 #' @param analyse analysis function. See \code{\link{runSimulation}}
@@ -66,13 +79,38 @@
 #'
 #' @param save logical; store temporary file in case of crashes. If detected
 #'   in the working directory will automatically be loaded to resume (see
-#'   \code{\link{runSimulation}} for similar behavior)
+#'   \code{\link{runSimulation}} for similar behaviour)
+#'
+#' @param resume logical; if a temporary \code{SimDesign} file is detected should
+#'   the simulation resume from this location? Keeping this \code{TRUE} is generally
+#'   recommended, however this should be disabled
+#'   if using \code{SimSolve} within \code{\link{runSimulation}} to avoid
+#'   reading improper save states
 #'
 #' @param verbose logical; print information to the console?
 #'
+#' @param check.interval logical; should an initial check be made to determine
+#'    whether \code{f(interval[1L])} and \code{f(interval[2L])} have opposite
+#'    signs? If \code{FALSE}, the specified \code{interval} is assumed to contain a root,
+#'    where \code{f(interval[1]) < 0} and \code{f(interval[2] > 0}. Default is \code{TRUE}
+#'
+#' @param predCI advertised confidence interval probability for final
+#'   model-based prediction of target \code{b} given the root input estimate.
+#'   Returned as an element in the \code{summary()} list output
+#'
 #' @param control a \code{list} of the algorithm control parameters. If not specified,
 #'   the defaults described below are used.
 #'
+#' @param wait.time (optional) argument passed to \code{\link{PBA}} to indicate
+#'   the time to wait (specified in minutes) per row in the \code{Design} object
+#'   rather than using pre-determined termination criteria based on the estimates.
+#'   For example, if three three conditions were defined in
+#'   \code{Design}, and \code{wait.time=5},
+#'   then the total search time till terminate after 15 minutes regardless of
+#'   independently specified termination criteria in \code{control}. Note that
+#'   \code{maxiter} is still used alongside \code{wait.time}, therefore this should
+#'   be increased as well (e.g., to \code{maxiter = 1000})
+#'
 #' \describe{
 #'    \item{\code{tol}}{tolerance criteria for early termination (.1 for
 #'      \code{integer = TRUE} searches; .00025 for non-integer searches}
@@ -81,13 +119,9 @@
 #'      \code{tol} criteria. Consecutive failures add -1 to the counter (default is 3)}
 #'    \item{\code{bolster}}{logical; should the PBA evaluations use bolstering based on previous
 #'      evaluations? Default is \code{TRUE}, though only applicable when \code{integer = TRUE} }
-#'    \item{\code{single_step.iter}}{when \code{integer = TRUE}, do not take steps larger than
-#'      size 1% the current value after this iteration number
-#'      (default is 40). This prevents wide oscillations
-#'      around the probable root for uncertain solutions}
 #'    \item{\code{interpolate.R}}{number of replications to collect prior to performing
 #'      the interpolation step (default is 3000 after accounting for data exclusion
-#'      from \code{burnin}). Setting this to 0 will disable any
+#'      from \code{burnin.iter}). Setting this to 0 will disable any
 #'      interpolation computations}
 #'    \item{\code{include_reps}}{logical; include a column in the \code{condition}
 #'      elements to indicate how many replications are currently being evaluated? Mainly
@@ -138,6 +172,8 @@
 #' @return the filled-in \code{design} object containing the associated lower and upper interval
 #'   estimates from the stochastic optimization
 #'
+#' @seealso \code{\link{SFA}}
+#'
 #' @export
 #'
 #' @references
@@ -152,7 +188,11 @@
 #'
 #' \dontrun{
 #'
-#' # TASK: Find specific sample size in each group for independent t-test
+#' ##########################
+#' ## A Priori Power Analysis
+#' ##########################
+#'
+#' # GOAL: Find specific sample size in each group for independent t-test
 #' # corresponding to a power rate of .8
 #' #
 #' # For ease of the setup, assume the groups are the same size, and the mean
@@ -193,16 +233,21 @@
 #'     # Must return a single number corresponding to f(x) in the
 #'     # root equation f(x) = b
 #'
-#'     ret <- EDR(results, alpha = condition$sig.level)
+#'     ret <- c(power = EDR(results, alpha = condition$sig.level))
 #'     ret
 #' }
 #'
 #' #~~~~~~~~~~~~~~~~~~~~~~~~
 #' #### Step 3 --- Optimize N over the rows in design
-#' # Initial search between N = [10,500] for each row using the default
-#' # integer solver (integer = TRUE)
 #'
-#' # In this example, b = target power
+#' # (For debugging) may want to see if simulation code works as intended first
+#' # for some given set of inputs
+#' runSimulation(design=createDesign(N=100, d=.8, sig.level=.05),
+#'               replications=10, generate=Generate, analyse=Analyse,
+#'               summarise=Summarise)
+#'
+#' # Initial search between N = [10,500] for each row using the default
+#'    # integer solver (integer = TRUE). In this example, b = target power
 #' solved <- SimSolve(design=Design, b=.8, interval=c(10, 500),
 #'                 generate=Generate, analyse=Analyse,
 #'                 summarise=Summarise)
@@ -219,24 +264,25 @@
 #'
 #' # verify with true power from pwr package
 #' library(pwr)
-#' pwr.t.test(d=.2, power = .8, sig.level = .05)
-#' pwr.t.test(d=.5, power = .8, sig.level = .05)
-#' pwr.t.test(d=.8, power = .8, sig.level = .05)
+#' pwr.t.test(d=.2, power = .8) # sig.level/alpha = .05 by default
+#' pwr.t.test(d=.5, power = .8)
+#' pwr.t.test(d=.8, power = .8)
 #'
 #' # use estimated N results to see how close power was
 #' N <- solved$N
-#' pwr.t.test(d=.2, n=N[1], sig.level = .05)
-#' pwr.t.test(d=.5, n=N[2], sig.level = .05)
-#' pwr.t.test(d=.8, n=N[3], sig.level = .05)
+#' pwr.t.test(d=.2, n=N[1])
+#' pwr.t.test(d=.5, n=N[2])
+#' pwr.t.test(d=.8, n=N[3])
 #'
 #' # with rounding
 #' N <- ceiling(solved$N)
-#' pwr.t.test(d=.2, n=N[1], sig.level = .05)
-#' pwr.t.test(d=.5, n=N[2], sig.level = .05)
-#' pwr.t.test(d=.8, n=N[3], sig.level = .05)
+#' pwr.t.test(d=.2, n=N[1])
+#' pwr.t.test(d=.5, n=N[2])
+#' pwr.t.test(d=.8, n=N[3])
 #'
 #' # failing analytic formula, confirm results with more precise
 #' #  simulation via runSimulation()
+#' #  (not required, if accuracy is important then ProBABLI should be run longer)
 #' csolved <- solved
 #' csolved$N <- ceiling(solved$N)
 #' confirm <- runSimulation(design=csolved, replications=10000, parallel=TRUE,
@@ -244,11 +290,37 @@
 #'                          summarise=Summarise)
 #' confirm
 #'
+#' # Alternatively, and more realistically, the wait.time argument can be used
+#' # to specify how long the user is willing to wait for a final estimate.
+#' # Solutions involving more iterations will be more accurate,
+#' # and therefore it is recommended to run the ProBABLI root-solver as long
+#' # the analyst can tolerate if the most accurate estimates are desired.
+#' # Below executes the simulation for 2 minutes for each condition up
+#' # to a maximum of 1000 iterations, terminating based on whichever occurs first
 #'
-#' ############
-#' # Similar setup as above, however goal is now to solve d given sample
-#' # size and power inputs (inputs for root no longer required to be an integer)
+#' solved_2min <- SimSolve(design=Design, b=.8, interval=c(10, 500),
+#'                 generate=Generate, analyse=Analyse, summarise=Summarise,
+#'                 wait.time=2, maxiter=1000)
+#' solved_2min
+#' summary(solved_2min)
 #'
+#' # use estimated N results to see how close power was
+#' N <- solved_2min$N
+#' pwr.t.test(d=.2, n=N[1])
+#' pwr.t.test(d=.5, n=N[2])
+#' pwr.t.test(d=.8, n=N[3])
+#'
+#' #------------------------------------------------
+#'
+#' #######################
+#' ## Sensitivity Analysis
+#' #######################
+#'
+#' # GOAL: solve effect size d given sample size and power inputs (inputs
+#' # for root no longer required to be an integer)
+#'
+#' # Generate-Analyse-Summarise functions identical to above, however
+#' # Design input includes NA for d element
 #' Design <- createDesign(N = c(100, 50, 25),
 #'                        d = NA,
 #'                        sig.level = .05)
@@ -264,8 +336,8 @@
 #' # In this example, b = target power
 #' # note that integer = FALSE to allow smooth updates of d
 #' solved <- SimSolve(design=Design, b = .8, interval=c(.1, 2),
-#'                 generate=Generate, analyse=Analyse,
-#'                 summarise=Summarise, integer=FALSE)
+#'                    generate=Generate, analyse=Analyse,
+#'                    summarise=Summarise, integer=FALSE)
 #' solved
 #' summary(solved)
 #' plot(solved, 1)
@@ -279,14 +351,65 @@
 #'
 #' # verify with true power from pwr package
 #' library(pwr)
-#' pwr.t.test(n=100, power = .8, sig.level = .05)
-#' pwr.t.test(n=50, power = .8, sig.level = .05)
-#' pwr.t.test(n=25, power = .8, sig.level = .05)
+#' pwr.t.test(n=100, power = .8)
+#' pwr.t.test(n=50, power = .8)
+#' pwr.t.test(n=25, power = .8)
 #'
 #' # use estimated d results to see how close power was
-#' pwr.t.test(n=100, d = solved$d[1], sig.level = .05)
-#' pwr.t.test(n=50, d = solved$d[2], sig.level = .05)
-#' pwr.t.test(n=25, d = solved$d[3], sig.level = .05)
+#' pwr.t.test(n=100, d = solved$d[1])
+#' pwr.t.test(n=50, d = solved$d[2])
+#' pwr.t.test(n=25, d = solved$d[3])
+#'
+#' # failing analytic formula, confirm results with more precise
+#' #  simulation via runSimulation()
+#' confirm <- runSimulation(design=solved, replications=10000, parallel=TRUE,
+#'                          generate=Generate, analyse=Analyse,
+#'                          summarise=Summarise)
+#' confirm
+#'
+#'
+#' #------------------------------------------------
+#'
+#' #####################
+#' ## Criterion Analysis
+#' #####################
+#'
+#' # GOAL: solve Type I error rate (alpha) given sample size, effect size, and
+#' # power inputs (inputs for root no longer required to be an integer). Only useful
+#' # when Type I error is less important than achieving the desired 1-beta (power)
+#'
+#' Design <- createDesign(N = 50,
+#'                         d = c(.2, .5, .8),
+#'                         sig.level = NA)
+#' Design    # solve for NA's
+#'
+#' # all other function definitions same as above
+#'
+#' # search for alpha within [.0001, .8]
+#' solved <- SimSolve(design=Design, b = .8, interval=c(.0001, .8),
+#'                    generate=Generate, analyse=Analyse,
+#'                    summarise=Summarise, integer=FALSE)
+#' solved
+#' summary(solved)
+#' plot(solved, 1)
+#' plot(solved, 2)
+#' plot(solved, 3)
+#'
+#' # plot median history and estimate precision
+#' plot(solved, 1, type = 'history')
+#' plot(solved, 1, type = 'density')
+#' plot(solved, 1, type = 'iterations')
+#'
+#' # verify with true power from pwr package
+#' library(pwr)
+#' pwr.t.test(n=50, power = .8, d = .2, sig.level=NULL)
+#' pwr.t.test(n=50, power = .8, d = .5, sig.level=NULL)
+#' pwr.t.test(n=50, power = .8, d = .8, sig.level=NULL)
+#'
+#' # use estimated alpha results to see how close power was
+#' pwr.t.test(n=50, d = .2, sig.level=solved$sig.level[1])
+#' pwr.t.test(n=50, d = .5, sig.level=solved$sig.level[2])
+#' pwr.t.test(n=50, d = .8, sig.level=solved$sig.level[3])
 #'
 #' # failing analytic formula, confirm results with more precise
 #' #  simulation via runSimulation()
@@ -297,26 +420,40 @@
 #'
 #' }
 SimSolve <- function(design, interval, b, generate, analyse, summarise,
-                     replications = list(burnin = 15L, burnin.reps = 100L,
-                                         max.reps = 500L, increase.by = 10L),
+                     replications = list(burnin.iter = 15L, burnin.reps = 100L,
+                                         max.reps = 500L, min.total.reps = 9000L,
+                                         increase.by = 10L),
                      integer = TRUE, formula = y ~ poly(x, 2), family = 'binomial',
-                     parallel = FALSE, cl = NULL, save = TRUE,
+                     parallel = FALSE, cl = NULL, save = TRUE, resume = TRUE,
+                     method = 'ProBABLI', wait.time = NULL,
                      ncores = parallel::detectCores() - 1L,
                      type = ifelse(.Platform$OS.type == 'windows', 'PSOCK', 'FORK'),
-                     maxiter = 100L, verbose = TRUE, control = list(), ...){
+                     maxiter = 100L, check.interval = TRUE,
+                     verbose = TRUE, control = list(), predCI = .95, ...){
 
     # robust <- FALSE
-    burnin <- 15L
+    if(is.null(control$print_RAM)) control$print_RAM <- FALSE
+    burnin.iter <- 15L
     if(is.list(replications)){
-        if(is.null(replications$burnin)) replications$burnin <- burnin else
-            burnin <- replications$burnin
+        if(is.null(replications$burnin.iter)) replications$burnin.iter <- burnin.iter else
+            burnin.iter <- replications$burnin.iter
         if(is.null(replications$burnin.reps)) replications$burnin.reps <- 100L
         if(is.null(replications$max.reps)) replications$max.reps <- 500L
         if(is.null(replications$increase.by)) replications$increase.by <- 10L
+        min.total.reps <- ifelse(is.null(replications$min.total.reps),
+                                 7500L, replications$min.total.reps)
         replications <- with(replications,
-                             pmin(max.reps, c(rep(burnin.reps, burnin),
+                             pmin(max.reps, c(rep(burnin.reps, burnin.iter),
                                               seq(burnin.reps, by=increase.by,
-                                                  length.out=maxiter-burnin))))
+                                                  length.out=maxiter-burnin.iter))))
+    }
+    if(method %in% c('Brent', 'bisection')){
+        stopifnot('replications must be a constant for root solver' =
+                      is.numeric(replications))
+        min.total.reps <- min(replications)
+        if(length(replications) == 1L)
+            replications <- rep(replications, maxiter)
+        stopifnot(length(replications) == maxiter)
     }
     ANALYSE_FUNCTIONS <- GENERATE_FUNCTIONS <- NULL
     .SIMDENV$ANALYSE_FUNCTIONS <- ANALYSE_FUNCTIONS <- analyse
@@ -331,7 +468,6 @@ SimSolve <- function(design, interval, b, generate, analyse, summarise,
     if(is.null(control$k.sucess)) control$k.success <- 3L
     if(is.null(control$interpolate.R)) control$interpolate.R <- 3000L
     if(is.null(control$bolster)) control$bolster <- TRUE
-    if(is.null(control$single_step.iter)) control$single_step.iter <- 40L
     if(is.null(control$include_reps)) control$include_reps <- FALSE
     on.exit(.SIMDENV$stored_results <- .SIMDENV$stored_medhistory <-
                 .SIMDENV$stored_history <- .SIMDENV$include_reps <- NULL,
@@ -351,26 +487,31 @@ SimSolve <- function(design, interval, b, generate, analyse, summarise,
         parallel <- useFuture <- FALSE
     }
 
+    stopifnot(method %in% c('ProBABLI', 'Brent', 'bisection'))
     stopifnot(!missing(b))
     stopifnot(length(b) == 1L)
     stopifnot(!missing(interval))
     if(length(replications) == 1L) replications <- rep(replications, maxiter)
     stopifnot(length(replications) == maxiter)
-    interpolate <- control$interpolate.R > 0L
-    interpolate.after <- max(which(cumsum(replications) <=
-                                       control$interpolate.R)) + 1L
+    if(method == 'ProBABLI'){
+        interpolate <- control$interpolate.R > 0L
+        interpolate.after <- max(which(cumsum(replications) <=
+                                           control$interpolate.R)) + 1L
+    } else interpolate <- interpolate.after <- NULL
     ReturnSimSolveInternals <- FALSE
     if(!is.null(attr(verbose, 'ReturnSimSolveInternals')))
         ReturnSimSolveInternals <- TRUE
 
-    root.fun <- function(x, b, design.row, replications, store = TRUE, ...){
+    root.fun <- function(x, b, design.row, replications, store = TRUE, integer = FALSE, ...){
+        if(integer) x <- as.integer(x)
         design.row[1L, which(is.na(design.row))] <- x
         if(.SIMDENV$include_reps) design.row$REPLICATIONS <- replications
         attr(design.row, 'SimSolve') <- TRUE
         ret <- runSimulation(design=design.row, replications=replications,
                              generate=generate, analyse=analyse,
-                             parallel=parallel, cl=cl,
-                             summarise=summarise, save=FALSE, verbose=FALSE, ...)
+                             summarise=summarise, parallel=parallel, cl=cl,
+                             save=FALSE, resume=FALSE, verbose=FALSE,
+                             control=.SIMDENV$FromSimSolve$control, ...)
         val <- ifelse(is.list(ret), ret[[1L]], ret[1L])
         if(store){
             pick <- min(which(sapply(.SIMDENV$stored_results, is.null)))
@@ -427,11 +568,12 @@ SimSolve <- function(design, interval, b, generate, analyse, summarise,
     compname <- Sys.info()['nodename']
     tmpfilename <- paste0('SIMSOLVE-TEMPFILE_', compname, '.rds')
     start <- 1L
-    if(file.exists(tmpfilename)){
+    if(resume && file.exists(tmpfilename)){
         roots <- readRDS(tmpfilename)
         start <- min(which(sapply(roots, is.null)))
         if(verbose)
-            message(paste0('\nContinuing SimSolve() run at design row ', start))
+            message(paste0('\nContinuing at design row ', start,
+                           '. If not intended, terminate and use SimClean()'))
     }
     for(i in start:nrow(design)){
         if(verbose){
@@ -439,6 +581,7 @@ SimSolve <- function(design, interval, b, generate, analyse, summarise,
             print(cbind(as.data.frame(design[i,]), b = b))
             cat("\n")
         }
+
         .SIMDENV$stored_results <- vector('list', maxiter)
         .SIMDENV$stored_medhistory <- rep(NA, maxiter)
         .SIMDENV$stored_history <- vector('list', maxiter)
@@ -448,24 +591,49 @@ SimSolve <- function(design, interval, b, generate, analyse, summarise,
                                       family=family,
                                       formula=formula,
                                       replications=replications,
+                                      min.total.reps=min.total.reps,
                                       tol=control$tol,
                                       rel.tol=control$rel.tol,
                                       b=b,
                                       bolster=control$bolster,
                                       k.success=control$k.success,
-                                      single_step.iter=control$single_step.iter,
                                       control=control,
                                       # robust = robust,
-                                      interpolate.burnin=burnin)
-        roots[[i]] <- try(PBA(root.fun, interval=interval[i, , drop=TRUE], b=b,
-                          design.row=as.data.frame(design[i,]),
-                          integer=integer, verbose=verbose, maxiter=maxiter, ...))
-        if(is(roots[[i]], 'try-error')){
-            is_below <- grepl("*below*", as.character(roots[[i]]))
-            if(is_below || grepl("*above*", as.character(roots[[i]])))
-                roots[[i]] <- list(root=ifelse(is_below, Inf, -Inf))
-            else roots[[i]] <- list(root=NA)
-            next
+                                      predCI = c((1-predCI)/2, predCI + (1-predCI)/2),
+                                      interpolate.burnin=burnin.iter)
+        if(method == 'ProBABLI'){
+            roots[[i]] <- try(PBA(root.fun, interval=interval[i, , drop=TRUE], b=b,
+                                  design.row=as.data.frame(design[i,]),
+                                  integer=integer, verbose=verbose, maxiter=maxiter,
+                                  miniter=1L, wait.time=wait.time, check.interval=check.interval,
+                                  ...), TRUE)
+            if(is(roots[[i]], 'try-error')){
+                is_below <- grepl("*below*", as.character(roots[[i]]))
+                if(is_below || grepl("*above*", as.character(roots[[i]])))
+                    roots[[i]] <- list(root=ifelse(is_below, Inf, -Inf))
+                else roots[[i]] <- list(root=NA)
+                next
+            }
+        } else if(method == 'Brent'){
+            time <- system.time(roots[[i]] <- stats::uniroot(root.fun,
+                                         interval=interval[i, , drop=TRUE], b=b,
+                                         design.row=as.data.frame(design[i,]),
+                                         integer=integer, replications=replications[i],
+                                         tol=control$tol, maxiter=maxiter, ...))
+            roots[[i]]$init.it <- roots[[i]]$estim.prec <- NULL
+            roots[[i]]$integer <- integer
+            roots[[i]]$total.replications <- roots[[i]]$iter * replications[1L]
+            roots[[i]]$time <- unname(time[1L])
+        } else if(method == 'bisection'){
+            time <- system.time(roots[[i]] <- bisection(root.fun,
+                                    interval=interval[i, , drop=TRUE], b=b,
+                                    design.row=as.data.frame(design[i,]),
+                                    integer=integer, replications=replications[i],
+                                    tol=control$tol, maxiter=maxiter,
+                                    check=check.interval, ...))
+            roots[[i]]$integer <- integer
+            roots[[i]]$total.replications <- roots[[i]]$iter * replications[1L]
+            roots[[i]]$time <- unname(time[1L])
         }
         tab <- .SIMDENV$stored_history[!sapply(.SIMDENV$stored_history, is.null)]
         if(ReturnSimSolveInternals) return(tab)
@@ -473,7 +641,7 @@ SimSolve <- function(design, interval, b, generate, analyse, summarise,
         if(verbose){
             cat("\r")
             tmp <- as.data.frame(design[i,])
-            cat(sprintf("\nSolution for %s: %.3f",
+            cat(sprintf(paste0("\nSolution for %s: %", if(integer) ".1f" else ".3f"),
                 colnames(design)[which(is.na(tmp))], roots[[i]]$root))
         }
         if(save && i < nrow(design)) saveRDS(roots, tmpfilename)
@@ -485,6 +653,7 @@ SimSolve <- function(design, interval, b, generate, analyse, summarise,
     attr(ret, 'roots') <- roots
     attr(ret, 'summarise_fun') <- summarise
     attr(ret, 'solve_name') <- solve_name
+    attr(ret, 'b') <- b
     class(ret) <- c('SimSolve', class(ret))
     ret
 }
@@ -508,7 +677,8 @@ summary.SimSolve <- function(object, tab.only = FALSE, reps.cutoff = 300, ...)
         attr(ret[[i]], "stored_tab") <- NULL
         ret[[i]]$tab <- stored_tab[[i]]
     }
-    names(ret) <- paste0('DesignRow_', 1L:length(ret))
+    names(ret) <- paste0('condition_', 1L:length(ret))
+    if(length(ret) == 1L) ret <- ret[[1L]]
     ret
 }
 
@@ -519,6 +689,7 @@ summary.SimSolve <- function(object, tab.only = FALSE, reps.cutoff = 300, ...)
 plot.SimSolve <- function(x, y, ...)
 {
     if(missing(y)) y <- 1L
+    b <- attr(x, 'b')
     roots <- attr(x, 'roots')[[y]]
     solve_name <- attr(x, 'solve_name')[y]
     dots <- list(...)
@@ -531,13 +702,20 @@ plot.SimSolve <- function(x, y, ...)
             tab <- tab[-c(1:so[[y]]$burnin), ]
         }
         if(dots$type == 'density')
-            with(tab, plot(density(x, weights=reps/sum(reps)),
-                           main = 'Density Using Replication Weights', las=1))
-        else
+            with(tab, plot(density(x, weights=reps/sum(reps)), xlab = solve_name,
+                           main = 'Density using replication weights', las=1))
+        else {
             with(tab, symbols(x, y, circles=sqrt(1 /reps/sum(reps)),
                               inches=0.2, fg="white", bg="black", las=1,
                               ylab = 'Summarise', xlab = solve_name,
                               main = 'Inverse replication weights'))
-    } else plot(roots, las=1, ...)
+            abline(h=b, lty=2, col='red')
+            abline(v=roots$root, lty=2, col='blue')
+        }
+    } else {
+        if(!is(roots, 'PBA'))
+            stop('No plot method supported for deterministic root-finder', call.=FALSE)
+        plot(roots, las=1, ...)
+    }
     return(invisible(NULL))
 }
