@@ -77,6 +77,13 @@ files[grepl('my-simp', files)]
 ## ----include=FALSE------------------------------------------------------------
 SimClean('my-simple-sim.rds')
 
+## -----------------------------------------------------------------------------
+# store_results=TRUE by default
+res <- runSimulation(Design, replications = 3, 
+              generate=Generate, analyse=Analyse, summarise=Summarise)
+results <- SimExtract(res, what = 'results')
+results
+
 ## -----------------------------------------------------------------------------
 res <- runSimulation(Design, replications = 1000, save_results=TRUE,
                      generate=Generate, analyse=Analyse, summarise=Summarise)
@@ -92,7 +99,8 @@ row1$condition
 head(row1$results)
 
 ## -----------------------------------------------------------------------------
-row1 <- SimResults(res, 1)
+# first row
+row1 <- SimResults(res, which = 1)
 str(row1)
 
 ## -----------------------------------------------------------------------------
@@ -102,9 +110,3 @@ str(input)
 ## -----------------------------------------------------------------------------
 SimClean(results = TRUE)
 
-## -----------------------------------------------------------------------------
-res <- runSimulation(Design, replications = 3, store_results=TRUE,
-              generate=Generate, analyse=Analyse, summarise=Summarise)
-list_results <- SimExtract(res, what = 'results')
-list_results
-
